import assert from 'assert';

import { EntityTransport } from '../src/';

import startRouter from './helpers/startRouter';

describe('Entity Transport add custom route', () => {
    let httpBackendTransport, zeronodeBackendTransport, kitooBackendTransport, httpBackend, znodeBackend, kitooBackend, httpClient, zeronodeClient, kitooClient;

    before ( async () => {
      try {
        httpBackendTransport = new EntityTransport(EntityTransport.HTTP);
        zeronodeBackendTransport = new EntityTransport(EntityTransport.ZERONODE);
        kitooBackendTransport = new EntityTransport(EntityTransport.KITOO, true);
        await startRouter('tcp://127.0.0.1:4666')

        znodeBackend = await zeronodeBackendTransport.bind({address: 'tcp://127.0.0.1:4601'});
        kitooBackend = await kitooBackendTransport.bind({router: 'tcp://127.0.0.1:4666', name: 'kitoo-client-1' });
        httpBackend = await httpBackendTransport.bind({ host: '127.0.0.1', port: 3088 });
        // ===========================


        const httpClientTransport = new EntityTransport(EntityTransport.HTTP);
        httpClient = await httpClientTransport.connect({host: 'http://127.0.0.1', port: 3088});

        const zeronodeClientTransport = new EntityTransport(EntityTransport.ZERONODE);
        zeronodeClient = await zeronodeClientTransport.connect({host: 'http://127.0.0.1', port: 4601});

        const kitooClientTransport = new EntityTransport(EntityTransport.KITOO, true);
        kitooClient = await kitooClientTransport.connect({host: '127.0.0.1', port: 4666, name: 'kitoo-client-1' });
      } catch (e) {
        console.error(e)
      }
    });

    describe ('Custom HTTP request', () => {
        it ('Should add new POST route', async () => {
            httpBackendTransport.addRoute({
              action: 'customRoute',
              route: '/customRoute',
              method: 'POST'
            });

            httpBackend.addMiddleware('customRoute', (req, res, next) => {
                const { payload } = httpBackend.parseRequest(req);

                assert.equal(payload.msg, 'Hello')
                res.json({ msg: 'World' });
            });

            const response = await httpClient.request({
              requestOptions: { route: '/customRoute' },
              payload: { msg: 'Hello' }
            });

            assert.equal(response.msg, 'World');
        });

        it ('Should add new GET route', async () => {
            httpBackendTransport.addRoute({
              action: 'customRouteGet',
              route: '/customRoute',
              method: 'GET'
            });

            httpBackend.addMiddleware('customRouteGet', (req, res, next) => {
                const { payload } = httpBackend.parseRequest(req);

                assert.equal(payload.msg, 'Hello')
                res.json({ msg: 'World' });
            });

            const response = await httpClient.request({
              requestOptions: { route: '/customRoute', method: 'GET' },
              payload: { msg: 'Hello' }
            });

            assert.equal(response.msg, 'World');
        });

      it ('Should add new DELETE route', async () => {
        httpBackendTransport.addRoute({
          action: 'customRouteDelete',
          route: '/customRoute',
          method: 'DELETE'
        });

        httpBackend.addMiddleware('customRouteDelete', (req, res, next) => {
          const { payload } = httpBackend.parseRequest(req);

          assert.equal(payload.msg, 'Hello')
          res.json({ msg: 'World' });
        });

        const response = await httpClient.request({
          requestOptions: { route: '/customRoute', method: 'DELETE' },
          payload: { msg: 'Hello' }
        });

        assert.equal(response.msg, 'World');
      });

      it ('Should add new PUT route', async () => {
        httpBackendTransport.addRoute({
          action: 'customRoutePut',
          route: '/customRoute',
          method: 'DELETE'
        });

        httpBackend.addMiddleware('customRoutePut', (req, res, next) => {
          const { payload } = httpBackend.parseRequest(req);

          assert.equal(payload.msg, 'Hello')
          res.json({ msg: 'World' });
        });

        const response = await httpClient.request({
          requestOptions: { route: '/customRoute', method: 'DELETE' },
          payload: { msg: 'Hello' }
        });

        assert.equal(response.msg, 'World');
      });


      it ('Should add new PUT route', async () => {
        httpBackendTransport.addRoute({
          action: 'customRoutePut',
          route: '/customRoute',
          method: 'DELETE'
        });

        httpBackend.addMiddleware('customRoutePut', (req, res, next) => {
          const { payload } = httpBackend.parseRequest(req);

          assert.equal(payload.msg, 'Hello')
          res.json({ msg: 'World' });
        });

        const response = await httpClient.request({
          requestOptions: { route: '/customRoute', method: 'DELETE' },
          payload: { msg: 'Hello' }
        });

        assert.equal(response.msg, 'World');
      });
    })

  describe ('Custom Zeronode request', () => {
      it ('Should add custom zeronode request handler', async () => {
        znodeBackend.addRoute({
          action: 'customRoute'
        });

        znodeBackend.addMiddleware('customRoute', (req, res, next) => {
          const { payload } = znodeBackend.parseRequest(req);
          assert.equal(payload.msg, 'Hello')
          res.json({ msg: 'World' });
        });

        const response = await zeronodeClient.request({
          action: 'customRoute',
          payload: { msg: 'Hello' }
        });

        assert.equal(response.msg, 'World');
      })
  })


  describe ('Custom Kitoo request', () => {
    it ('Should add custom kitoo-core request handler', async () => {
      kitooBackend.addRoute({
        action: 'customRoute'
      });

      kitooBackend.addMiddleware('customRoute', (req, res, next) => {
        const { payload } = kitooBackend.parseRequest(req);
        assert.equal(payload.msg, 'Hello')
        res.json({ msg: 'World' });
      });

      const response = await kitooClient.request({
        action: 'customRoute',
        payload: { msg: 'Hello' }
      });

      assert.equal(response.msg, 'World');
    })
  })
});