import { Schema } from 'mongoose';
import assert from 'assert';

import { Entity, EntityTransport } from '../src/';

describe('Entity HTTP Transport', () => {
  let httpBackendTransport, mongoTransport, mongoClient, httpBackend, httpClientTransport, UserEntity;

  before ( async () => {
    let UserSchema  = new Schema({
      name: String,
      age: Number,
      level: Number,
      instanceId: Number
    });

    // ** Mongo transport
    const schemas = [{
      name : 'users',
      schema: UserSchema
    }];

    httpBackendTransport = new EntityTransport(EntityTransport.HTTP);

    const mongoTransportOptions = { schemas, options: { debug: false }, url: 'mongodb://localhost/wirentity' };

    mongoTransport = new EntityTransport(EntityTransport.MONGO);

    // ** bind mongo backend and connect mongo client
    await mongoTransport.bind(mongoTransportOptions);
    mongoClient = await mongoTransport.connect();

    httpBackend = await httpBackendTransport.bind({ host: '127.0.0.1', port: 3008 });
    httpBackend.addMiddleware('runCommand', (req, res, next) => {
      const { payload } = httpBackend.parseRequest(req);
      payload.command ? next() : next({ status: 403, message: 'Forbidden'});
    });
    httpBackend.proxy(mongoClient);

    //===========================

    // ** now the client

    // ** make http transport for making http requests
    httpClientTransport = new EntityTransport(EntityTransport.HTTP);
    await httpClientTransport.connect({ host: 'http://127.0.0.1', port: 3008 });

    // ** create entity with this transport
    UserEntity = Entity.generateClass(httpClientTransport, 'users', '_id');
  });

  describe ('new UserEntity', () => {

    let user;

    it ('Should create new instance', async () => {
      await UserEntity.removeByQuery({});
      user = await UserEntity.create({ name: 'User1', age: 18, level: 10 });
      assert.equal(18, user.get('age'));
      assert.equal('User1', user.get('name'));
    });

    it ('Should change user\'s age', async () => {
      user.set('age', 19);
      await user.save();
      assert.equal(19, user.get('age'));

      const readUser = await UserEntity.loadById(user.getId());
      assert.equal(19, readUser.get('age'));
    });

    it ('Should load all users', async () => {
      const users = await UserEntity.loadAll();
      assert.equal(1, users.value().length);
    });

    it ('Should load user and select only age', async () => {
      const _user = await UserEntity.loadById(user.getId(), ['age']);
      assert.equal(19, _user.get('age'));
      assert.equal(undefined, _user.get('name'));
    });

    it ('Should remove user', async () => {
      let _user = await UserEntity.create({ name: 'User2', age: 18, level: 10 });
      assert.equal('User2', _user.get('name'));
      const userId = _user.getId();
      await UserEntity.removeById(userId);
      let __user = await UserEntity.loadById(userId);
      assert.equal(null, __user);
    });


    it ('Should load user by query', async () => {
      await UserEntity.removeByQuery({});
      await UserEntity.create({ name: 'Test_name 1', age: 18, level: 10, instanceId: 1 });
      await UserEntity.create({ name: 'Test_name 2', age: 19, level: 10, instanceId: 2 });
      await UserEntity.create({ name: 'Test_name 3', age: 20, level: 10, instanceId: 3 });

      let _user = await UserEntity.loadByQuery({ query: { instanceId: { $in: [1, 2, 3] } } });
      assert.equal(3, _user.value().length);
    });


    it ('Should update user by query', async () => {
      await UserEntity.update({ instanceId: 3 }, { age: 30 });
      let _user = await UserEntity.loadByQuery({ query: { instanceId: 3 } });
      assert.equal(30, _user.value()[0].age);
    });

    it ('Should load one user', async () => {
      let _user = await UserEntity.loadOne({ query: { level: 10 } });
      assert.equal(_user.get('level'), 10);
    });

    it ('Should count users collection', async () => {
      const count = await UserEntity.count({ age: 18 });
      const countAll = await UserEntity.count();
      assert.equal(count, 1);
      assert.equal(countAll, 3);
    });

    it ('Should sync user\'s data', async () => {
      const user = await UserEntity.loadOne();
      //update user's age
      await UserEntity.updateById(user.getId(), { age: 99 });
      const updatedUser = await UserEntity.loadById(user.getId());
      assert.notEqual(user.get('age'), updatedUser.get('age'));
      //sync users all data
      await user.sync();
      assert.equal(user.get('age'), updatedUser.get('age'));

      //update user's  age and name
      updatedUser.set({ age: 101, name: 'Updated name 1' });
      await updatedUser.save();
      assert.notEqual(user.get('age'), updatedUser.get('age'));
      assert.notEqual(user.get('name'), updatedUser.get('name'));

      //sync only age
      await user.sync(['age']);
      assert.equal(user.get('age'), updatedUser.get('age'));
      assert.notEqual(user.get('name'), updatedUser.get('name'));

      //sync name
      await user.sync(['name']);
      assert.equal(user.get('age'), updatedUser.get('age'));
      assert.equal(user.get('name'), updatedUser.get('name'));
    });

    it ('Should run aggregation on mongo', async () => {
        const [ response ] = await UserEntity.runCommand('aggregate', [{ $match: {} }, { $group: { _id: 1, sum: { $sum: '$age' } } }]);
        assert.equal(response.sum, 150);
    })

    it ('Should remove user by query', async () => {
      await UserEntity.removeByQuery({ level: 10 });
      let _user = await UserEntity.loadOne();
      assert.equal(_user, null);
    });

    it ('Should send 403 status', async () => {
      try  {
        await UserEntity.runCommand();
      } catch (e) {
        assert.equal(e.status, 403);
      }
    });

  })
});