import _ from 'lodash';
import Sequelize from 'sequelize';
import assert from 'assert';

import { Entity, EntityTransport } from '../src/';

describe('Postgres Entity', () => {
    let postgresTransport, UserEntity, postgresClient;

    before ( async () => {
        // ** Postgres transport
        const schemas = [{
            name : 'users',
            schema: {
                name: Sequelize.STRING,
                age: Sequelize.INTEGER,
                level: Sequelize.INTEGER,
                instanceId: Sequelize.INTEGER
            }
        }];

        const postgresTransportOptions = {
            schemas,
            options: {
              db: 'wire_entity_test2',
              username: 'postgres',
              password: '',
              logging: false
            }
        }

        postgresTransport = new EntityTransport(EntityTransport.POSTGRES);

        // ** bind postgres backend and connect postgres client
        await postgresTransport.bind(postgresTransportOptions);
        postgresClient = await postgresTransport.connect();

        // ** create entity with this transport
        UserEntity = Entity.generateClass(postgresTransport, 'users');
    });

    describe ('new UserEntity', () => {

        let user;

        it ('Should create new instance', async () => {
            await UserEntity.removeByQuery({});
            user = await UserEntity.create({ name: 'User1', age: 18, level: 10 });
            assert.equal(18, user.get('age'));
            assert.equal('User1', user.get('name'));
        });

        it ('Should change user\'s age', async () => {
            user.set('age', 19);
            await user.save();
            assert.equal(19, user.get('age'));

            const readUser = await UserEntity.loadById(user.getId());
            assert.equal(19, readUser.get('age'));
        });

        it ('Should load all users', async () => {
            const users = await UserEntity.loadAll();
            assert.equal(1, users.value().length);
        });

        it ('Should load user and select only age', async () => {
            const _user = await UserEntity.loadById(user.getId(), ['age']);
            assert.equal(19, _user.get('age'));
            assert.equal(undefined, _user.get('name'));
        });

        it ('Should remove user', async () => {
            let _user = await UserEntity.create({ name: 'User2', age: 18, level: 10 });
            assert.equal('User2', _user.get('name'));
            const userId = _user.getId();
            await UserEntity.removeById(userId);
            let __user = await UserEntity.loadById(userId);
            assert.equal(null, __user);
        });


        it ('Should load user by query', async () => {
            await UserEntity.removeByQuery({});
            await UserEntity.create({ name: 'Test_name 1', age: 18, level: 10, instanceId: 1 });
            await UserEntity.create({ name: 'Test_name 2', age: 19, level: 10, instanceId: 2 });
            await UserEntity.create({ name: 'Test_name 3', age: 20, level: 10, instanceId: 3 });

            let _user = await UserEntity.loadByQuery({ query: { instanceId: { $in: [1, 2, 3] } } });
            assert.equal(3, _user.value().length);
        });


        it ('Should update user by query', async () => {
            await UserEntity.update({ instanceId: 3 }, { age: 30 });
            let _user = await UserEntity.loadByQuery({ query: { instanceId: 3 } });
            assert.equal(30, _user.value()[0].age);
        });

        it ('Should load one user', async () => {
            let _user = await UserEntity.loadOne({ query: { level: 10 } });
            assert.equal(_user.get('level'), 10);
        });

        it ('Should count users collection', async () => {
            const count = await UserEntity.count({ age: 18 });
            const countAll = await UserEntity.count();
            assert.equal(count, 1);
            assert.equal(countAll, 3);
        });

        it ('Should sync user\'s data', async () => {
            const user = await UserEntity.loadOne();
            //update user's age
            await UserEntity.updateById(user.getId(), { age: 99 });
            const updatedUser = await UserEntity.loadById(user.getId());
            assert.notEqual(user.get('age'), updatedUser.get('age'));

            //sync users all data
            await user.sync();
            assert.equal(user.get('age'), updatedUser.get('age'));

            //update user's  age and name
            updatedUser.set({ age: 101, name: 'Updated name 1' });
            await updatedUser.save();
            assert.notEqual(user.get('age'), updatedUser.get('age'));
            assert.notEqual(user.get('name'), updatedUser.get('name'));

            //sync only age
            await user.sync(['age']);
            assert.equal(user.get('age'), updatedUser.get('age'));
            assert.notEqual(user.get('name'), updatedUser.get('name'));

            //sync name
            await user.sync(['name']);
            assert.equal(user.get('age'), updatedUser.get('age'));
            assert.equal(user.get('name'), updatedUser.get('name'));
        });

          it ('Should use sequelize option order', async () => {
            const users = await UserEntity.loadByQuery({ query: { }, order: [ ['age', 'DESC' ] ] });
            const clonnedUsers = JSON.parse(JSON.stringify(users.value()));
            clonnedUsers.sort((a, b) => b.age - a.age);
            assert.equal(_.map(users.value(), 'age').join(''), _.map(clonnedUsers, 'age').join(''));
          });

          it ('Should use sequelize option limit', async () => {
            const users = await UserEntity.loadByQuery({ query: { }, limit: 1 });
            assert.equal(users.length, 1);
          });

        it ('Should remove user by query', async () => {
            await UserEntity.removeByQuery({ level: 10 });
            await UserEntity.loadByQuery({ query: { level: 10 } });
            assert.equal(5, 5);
        });



    })
});
